/*****************************************************************************
 *
 * I2C Write Demo for PIC24.
 *
 *****************************************************************************
 * FileName:        main.c
 * Dependencies:    lcd.c
 * Processor:       PIC24FJ128
 * Compiler:       	C30
 * Linker:          
 * Company:         Microchip Technology Incorporated
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the "Company") is intended and supplied to you, the Company's
 * customer, for use solely and exclusively with products manufactured
 * by the Company. 
 *
 * The software is owned by the Company and/or its supplier, and is 
 * protected under applicable copyright laws. All rights are reserved. 
 * Any use in violation of the foregoing restrictions may subject the 
 * user to criminal sanctions under applicable laws, as well as to 
 * civil liability for the breach of the terms and conditions of this 
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES, 
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT, 
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Michael Javier      3/31/11      Updated for I2C code example
 *****************************************************************************/

#include <p24FJ128GA010.h>

_CONFIG1(JTAGEN_OFF & GCP_OFF & GWRP_OFF & BKBUG_OFF & ICS_PGx2 & FWDTEN_OFF)
_CONFIG2(IESO_OFF & FCKSM_CSDCMD & OSCIOFNC_ON &  FNOSC_PRI & POSCMOD_HS)

// Globally Defined Variables

#define MASTER 0x20						// defines the slave address of this controller
#define SLAVE 0x10						// defines the slave address to be controlled by this controller
#define TIMEOUT 0xfff					// defines the timeout value, must not be too small

unsigned char isRxReceived;				// this flag will indicate if all characters has been recieved and UART is idle again
unsigned char isLoading;				// this flag will indicate if UART has recieve a single character
unsigned char nBuffOutIdx;				// this index is use to fill buffer, indicator of how many character is loaded on the buffer.. ect

char I2CBuffer[16] = "                ";// this is the buffer and limited only to 16 bytes as LCD has 16 characeter per line only
char dummy;								// its a dummy variable usually use when disposing the read value
unsigned char bI2CRecieve;				// its a flag used to indicate that I2C is recieving from its master
unsigned char bI2CHasStarted;			// its a flag used to indicate that I2C is done recieving from its master and I2C has detected alredy the stop bit

// Globally Defined Methods
void OutToLCD();
void IOInit();
void I2CInit();
void UARTInit();
unsigned char I2C1Send();

int main()
{
	IOInit();				 //Initialize Switch and LED I/O	
	LCDInit();				 //Initialize PMP Module
	UARTInit();              //Initialize UART2 Module
	I2CInit();				 //Initialize I2C1 Module
	bI2CRecieve = 0;		 //Initialize flags and indeces
	bI2CHasStarted = 0;	
	isRxReceived = 0;
	isLoading = 0;
	nBuffOutIdx = 0;
	
	LCDL1Home();			 //Set LCD Home
    
	while(1)
	{
	
		if (!PORTDbits.RD6)				// poll for pressing S3 abcdefghijklmnop
		{	
			LCDClear();					// Clear LCD
			while(!PORTDbits.RD6);		// Will only allows the program to continue after releasing the switch
		}

		if (!PORTDbits.RD7)				// poll for pressing S6
		{	
			
			while(!PORTDbits.RD7);			// Will only allows the program to continue after releasing the switch
		}

		if( isLoading )
		{									// load character receive to output buffer
			char buffer; 
			unsigned char dly = 50;			// initialize delay duration local to this section
	
			while(!U2STAbits.TRMT);	
		
			if( nBuffOutIdx < 16 )			// read RX regbuffer if output buffer is not yet full
			{
				buffer = U2RXREG;	
				I2CBuffer[nBuffOutIdx] = buffer;
				nBuffOutIdx++;
			}
			else
				buffer = U2RXREG;			// discard when out of buffer range
		
			while ( dly)					// delay 205 cycle
			dly--;

			if( (U2STAbits.RIDLE))			// is receiver idle?	
			{
				isRxReceived = 1;			// if yes, output buffer to LCD
			}

			isLoading = 0;					// clear is loading flag

		}


		if( isRxReceived )
		{		
		
											// Output Buffer to LCD ; from lcd.c
			LCDClear();						// Clear LCD; from lcd.c
			LCDL1Home();					// set LCD to Line 1 start
			unsigned char nIndex = 0;		// reset Global Index
			
			while( nIndex < nBuffOutIdx )   // if index < buffer character count, output character to LCD
			{	
				unsigned short tout = 0;		
				LCDPut(I2CBuffer[nIndex]);  // send character to PMP to output it to LCD; from lcd.c
				U2TXREG = I2CBuffer[nIndex];// send back the character to TX buffer for transmission		
				while ( !U2STAbits.TRMT && tout < TIMEOUT)tout++;	// stay here while character at shift register is not yet transmitted		
				nIndex++;					// next character
			}	
			
			
			I2C1Send( );					// send recived data from buffer to I2C

			isRxReceived = 0;				// clear receive flag
			nBuffOutIdx = 0;			// reset output buffer index to 0
		
		}



		if( bI2CRecieve )
		{
	
			if( !I2C1STATbits.D_A && !I2C1STATbits.R_W)
			{		
				dummy = I2C1RCV; 				// address recieved
				nBuffOutIdx = 0;				// clear buffer index when slave address is recived
				I2C1CONbits.SCLREL = 1;
				bI2CHasStarted = 1;	
				
			}
			else if( I2C1STATbits.D_A && !I2C1STATbits.R_W)
			{
			
				I2CBuffer[nBuffOutIdx] = I2C1RCV & 0xff;	// data recieved then stored to buffer
				I2C1CONbits.SCLREL = 1;			// release the clock
				nBuffOutIdx++;					// increment buffer index


			}

			bI2CRecieve = 0;					// clear the section flag
		}

		if( bI2CHasStarted && I2C1STATbits.P )
		{

			 bI2CHasStarted = 0;			    // clear the section flag
			 OutToLCD();						// output to LCD + send to hyperterminal by UART TX
			 
		}
	

	}

return 0;
}

void OutToLCD()
{
	LCDL1Home();
	LCDClear();
	unsigned char i = 0;
	

	while( i < nBuffOutIdx )
	{	unsigned short tout = 0;
		LCDPut(I2CBuffer[i]);		
		U2TXREG = I2CBuffer[i];// send back the character to TX buffer for transmission		
		while ( !U2STAbits.TRMT && tout < TIMEOUT)tout++;	// stay here while character at shift register is not yet transmitted		
		i++;
	}



}

unsigned char I2C1Send( )
{
			unsigned char SlaveAdr; unsigned char len = 0;
			unsigned char to = 0;
			I2C1CONbits.SEN = 1;		// Set start bit
			
			
			while( !I2C1STATbits.S && (to < TIMEOUT) )to++;	// wait for start bit flag to set
			to = 0;

			I2C1TRN = (SLAVE << 1) & 0xFE;				// send (slave + write bit )		

			while ( I2C1STATbits.TRSTAT && (to < TIMEOUT))to++;	// wait for Transmit bit to be cleared
			to = 0;

			if( I2C1STATbits.ACKSTAT )
			{
				I2C1CONbits.PEN = 1;		// Set stop bit
				return 1;					// ret 1 if NAK
			}

			while ( len < nBuffOutIdx )
			{

				I2C1TRN = I2CBuffer[len];				// send data
				
				while ( I2C1STATbits.TRSTAT && (to < TIMEOUT) )to++;	// wait for Transmit bit to be cleared
				to = 0;
				len++;

				if( I2C1STATbits.ACKSTAT )
				{
					I2C1CONbits.PEN = 1;		// Set stop bit
					return 1;					// ret 1 if NAK
				}


			}	

			I2C1CONbits.PEN = 1;		// Set stop bit

			return 0;

}

void __attribute__((interrupt, shadow, no_auto_psv)) _SI2C1Interrupt()
{

	bI2CRecieve = 1;
	IFS1bits.SI2C1IF = 0;					// manually cleared SI2C1 Interrupt flag

}


void I2CInit()
{

	I2C1MSK = MASTER; 			// set this controller's device address

	I2C1CONbits.I2CEN = 1;		// enable I2C module
	I2C1CONbits.I2CSIDL = 0;	// Continue module operation in Idle mode
	//I2C1CONbits.SCLREL = ; 
	I2C1CONbits.IPMIEN = 0;		// IPMI mode disabled
	I2C1CONbits.A10M = 0;		// I2C1ADD is a 7-bit slave address
	I2C1CONbits.DISSLW = 0; 	// for 400 kHz up enable slew rate
	I2C1CONbits.SMEN = 1; 		// signal conditioning
	I2C1CONbits.GCEN = 0;  		// for slave mode only
	I2C1CONbits.STREN = 1;  	// Enable software or receive clock stretching, therefore clock must be release by slave when done on loading data to sync clock

	I2C1BRG = 3;				// use 1MHz SCL frequency but apparently, its frequency is 666 kHz

	IEC1bits.SI2C1IE = 1;		// enable Slave I2C1 interrupt
	IEC1bits.MI2C1IE = 1;		// enable Master I2C1 interrupt

}

void IOInit()
{
	TRISDbits.TRISD6 = 1;		//  PORTD<6> as input, SW3.
	TRISDbits.TRISD7 = 1;		//  PORTD<6> as input, SW4.
	AD1PCFG = 0xFFFF;
	TRISA = 0xFF00;	        	// PORTA<7:0> as output for LED.
}

void UARTInit()
{
	
	U2BRG = 25;					// 9600 

	U2MODEbits.UARTEN = 1;		// UART2 is Enabled
	U2MODEbits.USIDL = 0;		// Continue operation at Idlestate
	U2MODEbits.IREN = 0;		// IrDA En/Decoder is disabled
	U2MODEbits.RTSMD = 0; 		// flow control mode
	U2MODEbits.UEN = 0b10;		// UTX, RTX, U2CTS, U2RTS are enable and on use.
	U2MODEbits.WAKE = 1;		// Wake-up on start bit is enabled
	U2MODEbits.LPBACK = 0;		// Loop-back is disabled
	U2MODEbits.ABAUD = 0;		// auto baud is disabled
	U2MODEbits.RXINV = 0;		// No RX inversion
	U2MODEbits.BRGH = 0;		// low boud rate
	U2MODEbits.PDSEL = 0b00; 	// 8bit no parity
	U2MODEbits.STSEL = 0;		// one stop bit	
		

	U2STAbits.UTXISEL1 = 0b00;	
	//U2STAbits.TXINV = 0; 		// cant compile TXINV.. why???
	U2STA &= 0xDFFF;			// clear TXINV by bit masking
	U2STAbits.UTXBRK = 0;		// sync break tx is disabled
	U2STAbits.UTXEN = 1;		//transmit  is enabled
	U2STAbits.URXISEL = 0b00;	// interrupt flag bit is set when RXBUF is filled whith 1 character
	U2STAbits.ADDEN = 0;		// address detect mode is disabled

	IFS1bits.U2RXIF = 0;		// clear interrupt flag of rx
	IEC1bits.U2RXIE = 1;		// enable rx recieved data interrupt

	IFS1bits.U2TXIF = 0;		// clear interrupt flag of rx
	IEC1bits.U2TXIE = 1;		// enable rx recieved data interrupt


}

void __attribute__((interrupt, shadow, no_auto_psv)) _U2RXInterrupt()
{

	IFS1bits.U2RXIF = 0;					// manually cleared U2RX Interrupt flag
	isLoading = 1;							// signal main to store character recieve to buffer

}

void __attribute__((interrupt, shadow, no_auto_psv)) _U2TXInterrupt()
{

	IFS1bits.U2TXIF = 0;					// manually cleared U2RX Interrupt flag

}


void __attribute__((interrupt, shadow, no_auto_psv)) _MI2C1Interrupt()
{

	IFS1bits.MI2C1IF = 0;					// manually cleared MI2C1 Interrupt flag

}










